import asyncio
import concurrent.futures
import datetime
import gzip
import typing
from collections import deque
from typing import Deque, AsyncIterator, Union, List, Optional, Dict, Callable

import logging

import ydb
from .topic_writer import (
    PublicWriterSettings,
    WriterSettings,
    PublicMessage,
    PublicWriterInitInfo,
    InternalMessage,
    TopicWriterStopped,
    TopicWriterError,
    messages_to_proto_requests,
    PublicWriteResult,
    PublicWriteResultTypes,
    Message,
)
from .. import (
    _apis,
    issues,
)
from .._utilities import AtomicCounter
from .._errors import check_retriable_error
from .._topic_common import common as topic_common
from ..retries import RetrySettings
from .._grpc.grpcwrapper.ydb_topic_public_types import PublicCodec
from .._grpc.grpcwrapper.ydb_topic import (
    UpdateTokenRequest,
    UpdateTokenResponse,
    StreamWriteMessage,
    TransactionIdentity,
    WriterMessagesFromServerToClient,
)
from .._grpc.grpcwrapper.common_utils import (
    IGrpcWrapperAsyncIO,
    SupportedDriverType,
    GrpcWrapperAsyncIO,
)

from ..query.base import TxEvent

if typing.TYPE_CHECKING:
    from ..query.transaction import BaseQueryTxContext

from .._constants import DEFAULT_INITIAL_RESPONSE_TIMEOUT

logger = logging.getLogger(__name__)


class WriterAsyncIO:
    _loop: asyncio.AbstractEventLoop
    _reconnector: "WriterAsyncIOReconnector"
    _closed: bool
    _parent: typing.Any  # need for prevent close parent client by GC

    def __init__(
        self,
        driver: SupportedDriverType,
        settings: PublicWriterSettings,
        _client=None,
    ):
        self._loop = asyncio.get_running_loop()
        self._closed = False
        self._reconnector = WriterAsyncIOReconnector(driver=driver, settings=WriterSettings(settings))
        self._parent = _client

    async def __aenter__(self) -> "WriterAsyncIO":
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        try:
            await self.close()
        except BaseException:
            if exc_val is None:
                raise

    def __del__(self):
        if self._closed or self._loop.is_closed():
            return
        try:
            logger.debug("Topic writer was not closed properly. Consider using method close().")
            task = self._loop.create_task(self.close(flush=False))
            topic_common.wrap_set_name_for_asyncio_task(task, task_name="close writer")
        except BaseException:
            logger.warning("Something went wrong during writer close in __del__")

    async def close(self, *, flush: bool = True):
        if self._closed:
            return

        logger.debug("Close topic writer")
        self._closed = True

        await self._reconnector.close(flush)
        logger.debug("Topic writer was closed")

    async def write_with_ack(
        self,
        messages: Union[Message, List[Message]],
    ) -> Union[PublicWriteResultTypes, List[PublicWriteResultTypes]]:
        """
        IT IS SLOWLY WAY. IT IS BAD CHOISE IN MOST CASES.
        It is recommended to use write with optionally flush or write_with_ack_futures and receive acks by wait futures.

        send one or number of messages to server and wait acks.

        For wait with timeout use asyncio.wait_for.
        """
        logger.debug(
            "write_with_ack %s messages",
            len(messages) if isinstance(messages, list) else 1,
        )
        futures = await self.write_with_ack_future(messages)
        if not isinstance(futures, list):
            futures = [futures]

        await asyncio.wait(futures)
        results = [f.result() for f in futures]

        return results if isinstance(messages, list) else results[0]

    async def write_with_ack_future(
        self,
        messages: Union[Message, List[Message]],
    ) -> Union[asyncio.Future, List[asyncio.Future]]:
        """
        send one or number of messages to server.
        return feature, which can be waited for check send result.

        Usually it is fast method, but can wait if internal buffer is full.

        For wait with timeout use asyncio.wait_for.
        """
        logger.debug(
            "write_with_ack_future %s messages",
            len(messages) if isinstance(messages, list) else 1,
        )
        input_single_message = not isinstance(messages, list)
        converted_messages = []
        if isinstance(messages, list):
            for m in messages:
                converted_messages.append(PublicMessage._create_message(m))
        else:
            converted_messages = [PublicMessage._create_message(messages)]

        futures = await self._reconnector.write_with_ack_future(converted_messages)
        if input_single_message:
            return futures[0]
        else:
            return futures

    async def write(
        self,
        messages: Union[Message, List[Message]],
    ):
        """
        send one or number of messages to server.
        it put message to internal buffer

        For wait with timeout use asyncio.wait_for.
        """
        logger.debug(
            "write %s messages",
            len(messages) if isinstance(messages, list) else 1,
        )
        await self.write_with_ack_future(messages)

    async def flush(self):
        """
        Force send all messages from internal buffer and wait acks from server for all
        messages.

        For wait with timeout use asyncio.wait_for.
        """
        logger.debug("flush writer")
        return await self._reconnector.flush()

    async def wait_init(self) -> PublicWriterInitInfo:
        """
        wait while real connection will be established to server.

        For wait with timeout use asyncio.wait_for()
        """
        logger.debug("wait writer init")
        return await self._reconnector.wait_init()


class TxWriterAsyncIO(WriterAsyncIO):
    _tx: "BaseQueryTxContext"

    def __init__(
        self,
        tx: "BaseQueryTxContext",
        driver: SupportedDriverType,
        settings: PublicWriterSettings,
        _client=None,
        _is_implicit=False,
    ):
        self._tx = tx
        self._loop = asyncio.get_running_loop()
        self._closed = False
        self._reconnector = WriterAsyncIOReconnector(driver=driver, settings=WriterSettings(settings), tx=self._tx)
        self._parent = _client
        self._is_implicit = _is_implicit

        # For some reason, creating partition could conflict with other session operations.
        # Could be removed later.
        self._first_write = True

        tx._add_callback(TxEvent.BEFORE_COMMIT, self._on_before_commit, self._loop)
        tx._add_callback(TxEvent.BEFORE_ROLLBACK, self._on_before_rollback, self._loop)

    async def write(
        self,
        messages: Union[Message, List[Message]],
    ):
        """
        send one or number of messages to server.
        it put message to internal buffer

        For wait with timeout use asyncio.wait_for.
        """
        if self._first_write:
            self._first_write = False
            return await super().write_with_ack(messages)
        return await super().write(messages)

    async def _on_before_commit(self, tx: "BaseQueryTxContext"):
        if self._is_implicit:
            return
        await self.close()

    async def _on_before_rollback(self, tx: "BaseQueryTxContext"):
        if self._is_implicit:
            return
        await self.close(flush=False)


class WriterAsyncIOReconnector:
    _static_id_counter = AtomicCounter()

    _closed: bool
    _loop: asyncio.AbstractEventLoop
    _credentials: Union[ydb.credentials.Credentials, None]
    _driver: ydb.aio.Driver
    _init_message: StreamWriteMessage.InitRequest
    _stream_connected: asyncio.Event
    _settings: WriterSettings
    _codec: PublicCodec
    _codec_functions: Dict[PublicCodec, Callable[[bytes], bytes]]
    _encode_executor: Optional[concurrent.futures.Executor]
    _codec_selector_batch_num: int
    _codec_selector_last_codec: Optional[PublicCodec]
    _codec_selector_check_batches_interval: int
    _tx: Optional["BaseQueryTxContext"]

    if typing.TYPE_CHECKING:
        _messages_for_encode: asyncio.Queue[List[InternalMessage]]
    else:
        _messages_for_encode: asyncio.Queue
    _messages: Deque[InternalMessage]
    _messages_future: Deque[asyncio.Future]
    _new_messages: asyncio.Queue
    _background_tasks: List[asyncio.Task]

    _state_changed: asyncio.Event
    if typing.TYPE_CHECKING:
        _stop_reason: asyncio.Future[BaseException]
    else:
        _stop_reason: asyncio.Future
    _init_info: Optional[PublicWriterInitInfo]

    def __init__(
        self, driver: SupportedDriverType, settings: WriterSettings, tx: Optional["BaseQueryTxContext"] = None
    ):
        self._closed = False
        self._id = WriterAsyncIOReconnector._static_id_counter.inc_and_get()
        self._loop = asyncio.get_running_loop()
        self._driver = driver
        self._credentials = driver._credentials
        self._init_message = settings.create_init_request()
        self._new_messages = asyncio.Queue()
        self._init_info = None
        self._stream_connected = asyncio.Event()
        self._settings = settings
        self._tx = tx

        self._codec_functions = {
            PublicCodec.RAW: lambda data: data,
            PublicCodec.GZIP: gzip.compress,
        }

        if settings.encoders:
            self._codec_functions.update(settings.encoders)

        self._encode_executor = settings.encoder_executor

        self._codec_selector_batch_num = 0
        self._codec_selector_last_codec = None
        self._codec_selector_check_batches_interval = 10000

        self._codec = self._settings.codec
        if self._codec and self._codec not in self._codec_functions:
            known_codecs = sorted(self._codec_functions.keys())
            raise ValueError("Unknown codec for writer: %s, supported codecs: %s" % (self._codec, known_codecs))

        self._last_known_seq_no = 0
        self._messages_for_encode = asyncio.Queue()
        self._messages = deque()
        self._messages_future = deque()
        self._new_messages = asyncio.Queue()
        self._stop_reason = self._loop.create_future()
        self._background_tasks = [
            topic_common.wrap_set_name_for_asyncio_task(
                asyncio.create_task(self._connection_loop()),
                task_name="connection_loop",
            ),
            topic_common.wrap_set_name_for_asyncio_task(
                asyncio.create_task(self._encode_loop()),
                task_name="encode_loop",
            ),
        ]

        self._state_changed = asyncio.Event()
        logger.debug("init writer reconnector id=%s", self._id)

    async def close(self, flush: bool):
        if self._closed:
            return
        self._closed = True
        logger.debug("Close writer reconnector id=%s", self._id)

        if flush:
            await self.flush()

        self._stop(TopicWriterStopped())

        for task in self._background_tasks:
            task.cancel()
        await asyncio.wait(self._background_tasks)

        # if work was stopped before close by error - raise the error
        try:
            self._check_stop()
        except TopicWriterStopped:
            pass

        logger.debug("Writer reconnector id=%s was closed", self._id)

    async def wait_init(self) -> PublicWriterInitInfo:
        while True:
            if self._stop_reason.done():
                raise self._stop_reason.exception()

            if self._init_info:
                return self._init_info

            await self._state_changed.wait()

    async def wait_stop(self) -> BaseException:
        try:
            await self._stop_reason
        except BaseException as stop_reason:
            return stop_reason

    async def write_with_ack_future(self, messages: List[PublicMessage]) -> List[asyncio.Future]:
        # todo check internal buffer limit
        self._check_stop()

        if self._settings.auto_seqno:
            await self.wait_init()

        internal_messages = self._prepare_internal_messages(messages)
        messages_future = [self._loop.create_future() for _ in internal_messages]

        self._messages_future.extend(messages_future)

        if self._codec == PublicCodec.RAW:
            self._add_messages_to_send_queue(internal_messages)
        else:
            self._messages_for_encode.put_nowait(internal_messages)

        return messages_future

    def _add_messages_to_send_queue(self, internal_messages: List[InternalMessage]):
        self._messages.extend(internal_messages)
        for m in internal_messages:
            self._new_messages.put_nowait(m)

    def _prepare_internal_messages(self, messages: List[PublicMessage]) -> List[InternalMessage]:
        if self._settings.auto_created_at:
            now = datetime.datetime.now(datetime.timezone.utc)
        else:
            now = None

        res = []
        for m in messages:
            internal_message = InternalMessage(m)
            if self._settings.auto_seqno:
                if internal_message.seq_no is None:
                    self._last_known_seq_no += 1
                    internal_message.seq_no = self._last_known_seq_no
                else:
                    raise TopicWriterError("Explicit seqno and auto_seq setting is mutual exclusive")
            else:
                if internal_message.seq_no is None or internal_message.seq_no == 0:
                    raise TopicWriterError("Empty seqno and auto_seq setting is disabled")
                elif internal_message.seq_no <= self._last_known_seq_no:
                    raise TopicWriterError("Message seqno is duplicated: %s" % internal_message.seq_no)
                else:
                    self._last_known_seq_no = internal_message.seq_no

            if self._settings.auto_created_at:
                if internal_message.created_at is not None:
                    raise TopicWriterError(
                        "Explicit set auto_created_at and setting auto_created_at is mutual exclusive"
                    )
                else:
                    internal_message.created_at = now

            res.append(internal_message)

        return res

    def _check_stop(self):
        if self._stop_reason.done():
            raise self._stop_reason.exception()

    async def _connection_loop(self):
        retry_settings = RetrySettings(retry_cancelled=True)  # todo

        while True:
            attempt = 0  # todo calc and reset
            tasks = []

            # noinspection PyBroadException
            stream_writer = None
            try:
                logger.debug("writer reconnector %s connect attempt %s", self._id, attempt)
                tx_identity = None if self._tx is None else self._tx._tx_identity()
                stream_writer = await WriterAsyncIOStream.create(
                    self._driver,
                    self._init_message,
                    self._settings.update_token_interval,
                    tx_identity=tx_identity,
                )
                logger.debug(
                    "writer reconnector %s connected stream %s",
                    self._id,
                    stream_writer._id,
                )
                try:
                    if self._init_info is None:
                        self._last_known_seq_no = stream_writer.last_seqno
                        self._init_info = PublicWriterInitInfo(
                            last_seqno=stream_writer.last_seqno,
                            supported_codecs=stream_writer.supported_codecs,
                        )
                        self._state_changed.set()

                except asyncio.InvalidStateError:
                    pass

                self._stream_connected.set()

                send_loop = topic_common.wrap_set_name_for_asyncio_task(
                    asyncio.create_task(self._send_loop(stream_writer)),
                    task_name="writer send loop",
                )
                receive_loop = topic_common.wrap_set_name_for_asyncio_task(
                    asyncio.create_task(self._read_loop(stream_writer)),
                    task_name="writer receive loop",
                )

                tasks = [send_loop, receive_loop]
                done, _ = await asyncio.wait([send_loop, receive_loop], return_when=asyncio.FIRST_COMPLETED)
                done.pop().result()  # need for raise exception - reason of stop task
            except issues.Error as err:
                err_info = check_retriable_error(err, retry_settings, attempt)
                if not err_info.is_retriable or self._tx is not None:  # no retries in tx writer
                    logger.debug("writer reconnector %s stop connection loop due to %s", self._id, err)
                    self._stop(err)
                    return

                logger.debug(
                    "writer reconnector %s retry in %s seconds",
                    self._id,
                    err_info.sleep_timeout_seconds,
                )
                await asyncio.sleep(err_info.sleep_timeout_seconds)

            except (asyncio.CancelledError, Exception) as err:
                self._stop(err)
                return
            finally:
                for task in tasks:
                    task.cancel()
                if tasks:
                    await asyncio.wait(tasks)
                if stream_writer:
                    await stream_writer.close()

    async def _encode_loop(self):
        try:
            while True:
                messages = await self._messages_for_encode.get()
                while not self._messages_for_encode.empty():
                    messages.extend(self._messages_for_encode.get_nowait())

                logger.debug(
                    "writer reconnector %s encode %s messages",
                    self._id,
                    len(messages),
                )

                batch_codec = await self._codec_selector(messages)
                await self._encode_data_inplace(batch_codec, messages)
                self._add_messages_to_send_queue(messages)
        except BaseException as err:
            self._stop(err)

    async def _encode_data_inplace(self, codec: PublicCodec, messages: List[InternalMessage]):
        if codec == PublicCodec.RAW:
            return

        eventloop = asyncio.get_running_loop()
        encode_waiters = []
        encoder_function = self._codec_functions[codec]

        for message in messages:
            encoded_data_futures = eventloop.run_in_executor(
                self._encode_executor, encoder_function, message.get_data_bytes()
            )
            encode_waiters.append(encoded_data_futures)

        encoded_datas = await asyncio.gather(*encode_waiters)

        for index, data in enumerate(encoded_datas):
            message = messages[index]
            message.codec = codec
            message.data = data

    async def _codec_selector(self, messages: List[InternalMessage]) -> PublicCodec:
        if self._codec is not None:
            return self._codec

        if self._codec_selector_last_codec is None:
            available_codecs = await self._get_available_codecs()

            # use every of available encoders at start for prevent problems
            # with rare used encoders (on writer or reader side)
            if self._codec_selector_batch_num < len(available_codecs):
                codec = available_codecs[self._codec_selector_batch_num]
            else:
                codec = await self._codec_selector_by_check_compress(messages)
                self._codec_selector_last_codec = codec
        else:
            if self._codec_selector_batch_num % self._codec_selector_check_batches_interval == 0:
                self._codec_selector_last_codec = await self._codec_selector_by_check_compress(messages)
            codec = self._codec_selector_last_codec
        self._codec_selector_batch_num += 1
        return codec

    async def _get_available_codecs(self) -> List[PublicCodec]:
        info = await self.wait_init()
        topic_supported_codecs = info.supported_codecs
        if not topic_supported_codecs:
            topic_supported_codecs = [PublicCodec.RAW, PublicCodec.GZIP]

        res = []
        for codec in topic_supported_codecs:
            if codec in self._codec_functions:
                res.append(codec)

        if not res:
            raise TopicWriterError("Writer does not support topic's codecs")

        res.sort()

        return res

    async def _codec_selector_by_check_compress(self, messages: List[InternalMessage]) -> PublicCodec:
        """
        Try to compress messages and choose codec with the smallest result size.
        """

        test_messages = messages[:10]

        available_codecs = await self._get_available_codecs()
        if len(available_codecs) == 1:
            return available_codecs[0]

        def get_compressed_size(codec) -> int:
            s = 0
            f = self._codec_functions[codec]

            for m in test_messages:
                encoded = f(m.get_data_bytes())
                s += len(encoded)

            return s

        def select_codec() -> PublicCodec:
            min_codec = available_codecs[0]
            min_size = get_compressed_size(min_codec)
            for codec in available_codecs[1:]:
                size = get_compressed_size(codec)
                if size < min_size:
                    min_codec = codec
                    min_size = size
            return min_codec

        loop = asyncio.get_running_loop()
        codec = await loop.run_in_executor(self._encode_executor, select_codec)
        return codec

    async def _read_loop(self, writer: "WriterAsyncIOStream"):
        while True:
            resp = await writer.receive()

            logger.debug("writer reconnector %s received %s acks", self._id, len(resp.acks))

            for ack in resp.acks:
                self._handle_receive_ack(ack)

    def _handle_receive_ack(self, ack):
        current_message = self._messages.popleft()
        message_future = self._messages_future.popleft()
        if current_message.seq_no != ack.seq_no:
            raise TopicWriterError(
                "internal error - receive unexpected ack. Expected seqno: %s, received seqno: %s"
                % (current_message.seq_no, ack.seq_no)
            )
        write_ack_msg = StreamWriteMessage.WriteResponse.WriteAck
        status = ack.message_write_status
        if isinstance(status, write_ack_msg.StatusSkipped):
            result = PublicWriteResult.Skipped()
        elif isinstance(status, write_ack_msg.StatusWritten):
            result = PublicWriteResult.Written(offset=status.offset)
        elif isinstance(status, write_ack_msg.StatusWrittenInTx):
            result = PublicWriteResult.WrittenInTx()
        else:
            raise TopicWriterError("internal error - receive unexpected ack message.")
        message_future.set_result(result)
        logger.debug(
            "writer reconnector %s ack seqno=%s result=%s",
            self._id,
            ack.seq_no,
            type(result).__name__,
        )

    async def _send_loop(self, writer: "WriterAsyncIOStream"):
        try:
            logger.debug("writer reconnector %s send loop start", self._id)
            messages = list(self._messages)

            last_seq_no = 0
            for m in messages:
                writer.write([m])
                logger.debug(
                    "writer reconnector %s sent buffered message seqno=%s",
                    self._id,
                    m.seq_no,
                )
                last_seq_no = m.seq_no

            while True:
                m = await self._new_messages.get()  # type: InternalMessage
                if m.seq_no > last_seq_no:
                    writer.write([m])
                    logger.debug(
                        "writer reconnector %s sent message seqno=%s",
                        self._id,
                        m.seq_no,
                    )
        except asyncio.CancelledError:
            # the loop task cancelled be parent code, for example for reconnection
            # no need to stop all work.
            raise
        except BaseException as e:
            self._stop(e)
            raise

    def _stop(self, reason: BaseException):
        if reason is None:
            raise Exception("writer stop reason can not be None")

        if self._stop_reason.done():
            return

        self._stop_reason.set_exception(reason)

        for f in self._messages_future:
            f.set_exception(reason)

        self._state_changed.set()
        logger.info("Stop topic writer %s: %s" % (self._id, reason))

    async def flush(self):
        if not self._messages_future:
            return

        # wait last message
        await asyncio.wait(self._messages_future)


class WriterAsyncIOStream:
    _static_id_counter = AtomicCounter()

    # todo slots
    _closed: bool

    last_seqno: int
    supported_codecs: Optional[List[PublicCodec]]

    _stream: IGrpcWrapperAsyncIO
    _requests: asyncio.Queue
    _responses: AsyncIterator

    _update_token_interval: Optional[Union[int, float]]
    _update_token_task: Optional[asyncio.Task]
    _update_token_event: asyncio.Event
    _get_token_function: Optional[Callable[[], str]]

    _tx_identity: Optional[TransactionIdentity]

    def __init__(
        self,
        update_token_interval: Optional[Union[int, float]] = None,
        get_token_function: Optional[Callable[[], str]] = None,
        tx_identity: Optional[TransactionIdentity] = None,
    ):
        self._closed = False
        self._id = WriterAsyncIOStream._static_id_counter.inc_and_get()

        self._update_token_interval = update_token_interval
        self._get_token_function = get_token_function
        self._update_token_event = asyncio.Event()
        self._update_token_task = None

        self._tx_identity = tx_identity

    async def close(self):
        if self._closed:
            return
        self._closed = True
        logger.debug("writer stream %s close", self._id)

        if self._update_token_task:
            self._update_token_task.cancel()
            await asyncio.wait([self._update_token_task])

        self._stream.close()
        logger.debug("writer stream %s was closed", self._id)

    @staticmethod
    async def create(
        driver: SupportedDriverType,
        init_request: StreamWriteMessage.InitRequest,
        update_token_interval: Optional[Union[int, float]] = None,
        tx_identity: Optional[TransactionIdentity] = None,
    ) -> "WriterAsyncIOStream":
        stream = GrpcWrapperAsyncIO(StreamWriteMessage.FromServer.from_proto)

        await stream.start(driver, _apis.TopicService.Stub, _apis.TopicService.StreamWrite)

        creds = driver._credentials
        writer = WriterAsyncIOStream(
            update_token_interval=update_token_interval,
            get_token_function=creds.get_auth_token if creds else lambda: "",
            tx_identity=tx_identity,
        )
        await writer._start(stream, init_request)
        logger.debug(
            "writer stream %s started seqno=%s",
            writer._id,
            writer.last_seqno,
        )
        return writer

    async def receive(self) -> StreamWriteMessage.WriteResponse:
        while True:
            item = await self._stream.receive()

            if isinstance(item, StreamWriteMessage.WriteResponse):
                return item
            if isinstance(item, UpdateTokenResponse):
                self._update_token_event.set()
                continue

            # todo log unknown messages instead of raise exception
            raise Exception("Unknown message while read writer answers: %s" % item)

    async def _start(self, stream: IGrpcWrapperAsyncIO, init_message: StreamWriteMessage.InitRequest):
        logger.debug("writer stream %s send init request", self._id)
        stream.write(StreamWriteMessage.FromClient(init_message))

        try:
            resp = await stream.receive(timeout=DEFAULT_INITIAL_RESPONSE_TIMEOUT)
        except asyncio.TimeoutError:
            raise TopicWriterError("Timeout waiting for init response")

        self._ensure_ok(resp)
        if not isinstance(resp, StreamWriteMessage.InitResponse):
            raise TopicWriterError("Unexpected answer for init request: %s" % resp)

        self.last_seqno = resp.last_seq_no
        self.supported_codecs = [PublicCodec(codec) for codec in resp.supported_codecs]
        logger.debug(
            "writer stream %s init done last_seqno=%s",
            self._id,
            self.last_seqno,
        )

        self._stream = stream

        if self._update_token_interval is not None:
            self._update_token_event.set()
            self._update_token_task = topic_common.wrap_set_name_for_asyncio_task(
                asyncio.create_task(self._update_token_loop()),
                task_name="update_token_loop",
            )

    @staticmethod
    def _ensure_ok(message: WriterMessagesFromServerToClient):
        if not message.status.is_success():
            raise TopicWriterError(f"status error from server in writer: {message.status}")

    def write(self, messages: List[InternalMessage]):
        if self._closed:
            raise RuntimeError("Can not write on closed stream.")

        logger.debug("writer stream %s send %s messages", self._id, len(messages))

        for request in messages_to_proto_requests(messages, self._tx_identity):
            self._stream.write(request)

    async def _update_token_loop(self):
        while True:
            await asyncio.sleep(self._update_token_interval)
            token = self._get_token_function()
            if asyncio.iscoroutine(token):
                token = await token
            logger.debug("writer stream %s update token", self._id)
            await self._update_token(token=token)

    async def _update_token(self, token: str):
        await self._update_token_event.wait()
        try:
            msg = StreamWriteMessage.FromClient(UpdateTokenRequest(token))
            self._stream.write(msg)
            logger.debug("writer stream %s token sent", self._id)
        finally:
            self._update_token_event.clear()



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_scheme.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v3.protos import ydb_common_pb2 as protos_dot_ydb__common__pb2
from ydb._grpc.v3.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/ydb_scheme.proto',
  package='Ydb.Scheme',
  syntax='proto3',
  serialized_options=b'\n\025tech.ydb.proto.schemeB\025SchemeOperationProtosZ9github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Scheme\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17protos/ydb_scheme.proto\x12\nYdb.Scheme\x1a\x17protos/ydb_common.proto\x1a\x1aprotos/ydb_operation.proto\"_\n\x14MakeDirectoryRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"E\n\x15MakeDirectoryResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"a\n\x16RemoveDirectoryRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"G\n\x17RemoveDirectoryResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"_\n\x14ListDirectoryRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"E\n\x15ListDirectoryResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"8\n\x0bPermissions\x12\x0f\n\x07subject\x18\x01 \x01(\t\x12\x18\n\x10permission_names\x18\x02 \x03(\t\"\xc1\x04\n\x05\x45ntry\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05owner\x18\x02 \x01(\t\x12$\n\x04type\x18\x05 \x01(\x0e\x32\x16.Ydb.Scheme.Entry.Type\x12\x36\n\x15\x65\x66\x66\x65\x63tive_permissions\x18\x06 \x03(\x0b\x32\x17.Ydb.Scheme.Permissions\x12,\n\x0bpermissions\x18\x07 \x03(\x0b\x32\x17.Ydb.Scheme.Permissions\x12\x12\n\nsize_bytes\x18\x08 \x01(\x04\x12)\n\ncreated_at\x18\t \x01(\x0b\x32\x15.Ydb.VirtualTimestamp\"\xcf\x02\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\r\n\tDIRECTORY\x10\x01\x12\t\n\x05TABLE\x10\x02\x12\x14\n\x10PERS_QUEUE_GROUP\x10\x03\x12\x0c\n\x08\x44\x41TABASE\x10\x04\x12\x0f\n\x0bRTMR_VOLUME\x10\x05\x12\x16\n\x12\x42LOCK_STORE_VOLUME\x10\x06\x12\x15\n\x11\x43OORDINATION_NODE\x10\x07\x12\x10\n\x0c\x43OLUMN_STORE\x10\x0c\x12\x10\n\x0c\x43OLUMN_TABLE\x10\r\x12\x0c\n\x08SEQUENCE\x10\x0f\x12\x0f\n\x0bREPLICATION\x10\x10\x12\t\n\x05TOPIC\x10\x11\x12\x12\n\x0e\x45XTERNAL_TABLE\x10\x12\x12\x18\n\x14\x45XTERNAL_DATA_SOURCE\x10\x13\x12\x08\n\x04VIEW\x10\x14\x12\x11\n\rRESOURCE_POOL\x10\x15\x12\x0c\n\x08TRANSFER\x10\x17\x12\x0c\n\x08SYS_VIEW\x10\x18\"[\n\x13ListDirectoryResult\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x11.Ydb.Scheme.Entry\x12#\n\x08\x63hildren\x18\x02 \x03(\x0b\x32\x11.Ydb.Scheme.Entry\"^\n\x13\x44\x65scribePathRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"D\n\x14\x44\x65scribePathResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"5\n\x12\x44\x65scribePathResult\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x11.Ydb.Scheme.Entry\"\xb2\x01\n\x11PermissionsAction\x12(\n\x05grant\x18\x01 \x01(\x0b\x32\x17.Ydb.Scheme.PermissionsH\x00\x12)\n\x06revoke\x18\x02 \x01(\x0b\x32\x17.Ydb.Scheme.PermissionsH\x00\x12&\n\x03set\x18\x03 \x01(\x0b\x32\x17.Ydb.Scheme.PermissionsH\x00\x12\x16\n\x0c\x63hange_owner\x18\x04 \x01(\tH\x00\x42\x08\n\x06\x61\x63tion\"\xde\x01\n\x18ModifyPermissionsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12.\n\x07\x61\x63tions\x18\x03 \x03(\x0b\x32\x1d.Ydb.Scheme.PermissionsAction\x12\x19\n\x11\x63lear_permissions\x18\x04 \x01(\x08\x12\x1f\n\x15interrupt_inheritance\x18\x05 \x01(\x08H\x00\x42\r\n\x0binheritance\"I\n\x19ModifyPermissionsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.OperationBl\n\x15tech.ydb.proto.schemeB\x15SchemeOperationProtosZ9github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Scheme\xf8\x01\x01\x62\x06proto3'
  ,
  dependencies=[protos_dot_ydb__common__pb2.DESCRIPTOR,protos_dot_ydb__operation__pb2.DESCRIPTOR,])



_ENTRY_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='Ydb.Scheme.Entry.Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DIRECTORY', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TABLE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PERS_QUEUE_GROUP', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATABASE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RTMR_VOLUME', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BLOCK_STORE_VOLUME', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COORDINATION_NODE', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COLUMN_STORE', index=8, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COLUMN_TABLE', index=9, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SEQUENCE', index=10, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REPLICATION', index=11, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TOPIC', index=12, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXTERNAL_TABLE', index=13, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXTERNAL_DATA_SOURCE', index=14, number=19,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VIEW', index=15, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_POOL', index=16, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRANSFER', index=17, number=23,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SYS_VIEW', index=18, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=901,
  serialized_end=1236,
)
_sym_db.RegisterEnumDescriptor(_ENTRY_TYPE)


_MAKEDIRECTORYREQUEST = _descriptor.Descriptor(
  name='MakeDirectoryRequest',
  full_name='Ydb.Scheme.MakeDirectoryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation_params', full_name='Ydb.Scheme.MakeDirectoryRequest.operation_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='Ydb.Scheme.MakeDirectoryRequest.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=92,
  serialized_end=187,
)


_MAKEDIRECTORYRESPONSE = _descriptor.Descriptor(
  name='MakeDirectoryResponse',
  full_name='Ydb.Scheme.MakeDirectoryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='Ydb.Scheme.MakeDirectoryResponse.operation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=189,
  serialized_end=258,
)


_REMOVEDIRECTORYREQUEST = _descriptor.Descriptor(
  name='RemoveDirectoryRequest',
  full_name='Ydb.Scheme.RemoveDirectoryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation_params', full_name='Ydb.Scheme.RemoveDirectoryRequest.operation_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='Ydb.Scheme.RemoveDirectoryRequest.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=260,
  serialized_end=357,
)


_REMOVEDIRECTORYRESPONSE = _descriptor.Descriptor(
  name='RemoveDirectoryResponse',
  full_name='Ydb.Scheme.RemoveDirectoryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='Ydb.Scheme.RemoveDirectoryResponse.operation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=359,
  serialized_end=430,
)


_LISTDIRECTORYREQUEST = _descriptor.Descriptor(
  name='ListDirectoryRequest',
  full_name='Ydb.Scheme.ListDirectoryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation_params', full_name='Ydb.Scheme.ListDirectoryRequest.operation_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='Ydb.Scheme.ListDirectoryRequest.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=432,
  serialized_end=527,
)


_LISTDIRECTORYRESPONSE = _descriptor.Descriptor(
  name='ListDirectoryResponse',
  full_name='Ydb.Scheme.ListDirectoryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='Ydb.Scheme.ListDirectoryResponse.operation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=529,
  serialized_end=598,
)


_PERMISSIONS = _descriptor.Descriptor(
  name='Permissions',
  full_name='Ydb.Scheme.Permissions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='subject', full_name='Ydb.Scheme.Permissions.subject', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='permission_names', full_name='Ydb.Scheme.Permissions.permission_names', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=600,
  serialized_end=656,
)


_ENTRY = _descriptor.Descriptor(
  name='Entry',
  full_name='Ydb.Scheme.Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Ydb.Scheme.Entry.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='owner', full_name='Ydb.Scheme.Entry.owner', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='Ydb.Scheme.Entry.type', index=2,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='effective_permissions', full_name='Ydb.Scheme.Entry.effective_permissions', index=3,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='permissions', full_name='Ydb.Scheme.Entry.permissions', index=4,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='Ydb.Scheme.Entry.size_bytes', index=5,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='Ydb.Scheme.Entry.created_at', index=6,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ENTRY_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=659,
  serialized_end=1236,
)


_LISTDIRECTORYRESULT = _descriptor.Descriptor(
  name='ListDirectoryResult',
  full_name='Ydb.Scheme.ListDirectoryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='self', full_name='Ydb.Scheme.ListDirectoryResult.self', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='children', full_name='Ydb.Scheme.ListDirectoryResult.children', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1238,
  serialized_end=1329,
)


_DESCRIBEPATHREQUEST = _descriptor.Descriptor(
  name='DescribePathRequest',
  full_name='Ydb.Scheme.DescribePathRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation_params', full_name='Ydb.Scheme.DescribePathRequest.operation_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='Ydb.Scheme.DescribePathRequest.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1331,
  serialized_end=1425,
)


_DESCRIBEPATHRESPONSE = _descriptor.Descriptor(
  name='DescribePathResponse',
  full_name='Ydb.Scheme.DescribePathResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='Ydb.Scheme.DescribePathResponse.operation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1427,
  serialized_end=1495,
)


_DESCRIBEPATHRESULT = _descriptor.Descriptor(
  name='DescribePathResult',
  full_name='Ydb.Scheme.DescribePathResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='self', full_name='Ydb.Scheme.DescribePathResult.self', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1497,
  serialized_end=1550,
)


_PERMISSIONSACTION = _descriptor.Descriptor(
  name='PermissionsAction',
  full_name='Ydb.Scheme.PermissionsAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='grant', full_name='Ydb.Scheme.PermissionsAction.grant', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='revoke', full_name='Ydb.Scheme.PermissionsAction.revoke', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='set', full_name='Ydb.Scheme.PermissionsAction.set', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='change_owner', full_name='Ydb.Scheme.PermissionsAction.change_owner', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='action', full_name='Ydb.Scheme.PermissionsAction.action',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1553,
  serialized_end=1731,
)


_MODIFYPERMISSIONSREQUEST = _descriptor.Descriptor(
  name='ModifyPermissionsRequest',
  full_name='Ydb.Scheme.ModifyPermissionsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation_params', full_name='Ydb.Scheme.ModifyPermissionsRequest.operation_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='Ydb.Scheme.ModifyPermissionsRequest.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actions', full_name='Ydb.Scheme.ModifyPermissionsRequest.actions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clear_permissions', full_name='Ydb.Scheme.ModifyPermissionsRequest.clear_permissions', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interrupt_inheritance', full_name='Ydb.Scheme.ModifyPermissionsRequest.interrupt_inheritance', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='inheritance', full_name='Ydb.Scheme.ModifyPermissionsRequest.inheritance',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1734,
  serialized_end=1956,
)


_MODIFYPERMISSIONSRESPONSE = _descriptor.Descriptor(
  name='ModifyPermissionsResponse',
  full_name='Ydb.Scheme.ModifyPermissionsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='Ydb.Scheme.ModifyPermissionsResponse.operation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1958,
  serialized_end=2031,
)

_MAKEDIRECTORYREQUEST.fields_by_name['operation_params'].message_type = protos_dot_ydb__operation__pb2._OPERATIONPARAMS
_MAKEDIRECTORYRESPONSE.fields_by_name['operation'].message_type = protos_dot_ydb__operation__pb2._OPERATION
_REMOVEDIRECTORYREQUEST.fields_by_name['operation_params'].message_type = protos_dot_ydb__operation__pb2._OPERATIONPARAMS
_REMOVEDIRECTORYRESPONSE.fields_by_name['operation'].message_type = protos_dot_ydb__operation__pb2._OPERATION
_LISTDIRECTORYREQUEST.fields_by_name['operation_params'].message_type = protos_dot_ydb__operation__pb2._OPERATIONPARAMS
_LISTDIRECTORYRESPONSE.fields_by_name['operation'].message_type = protos_dot_ydb__operation__pb2._OPERATION
_ENTRY.fields_by_name['type'].enum_type = _ENTRY_TYPE
_ENTRY.fields_by_name['effective_permissions'].message_type = _PERMISSIONS
_ENTRY.fields_by_name['permissions'].message_type = _PERMISSIONS
_ENTRY.fields_by_name['created_at'].message_type = protos_dot_ydb__common__pb2._VIRTUALTIMESTAMP
_ENTRY_TYPE.containing_type = _ENTRY
_LISTDIRECTORYRESULT.fields_by_name['self'].message_type = _ENTRY
_LISTDIRECTORYRESULT.fields_by_name['children'].message_type = _ENTRY
_DESCRIBEPATHREQUEST.fields_by_name['operation_params'].message_type = protos_dot_ydb__operation__pb2._OPERATIONPARAMS
_DESCRIBEPATHRESPONSE.fields_by_name['operation'].message_type = protos_dot_ydb__operation__pb2._OPERATION
_DESCRIBEPATHRESULT.fields_by_name['self'].message_type = _ENTRY
_PERMISSIONSACTION.fields_by_name['grant'].message_type = _PERMISSIONS
_PERMISSIONSACTION.fields_by_name['revoke'].message_type = _PERMISSIONS
_PERMISSIONSACTION.fields_by_name['set'].message_type = _PERMISSIONS
_PERMISSIONSACTION.oneofs_by_name['action'].fields.append(
  _PERMISSIONSACTION.fields_by_name['grant'])
_PERMISSIONSACTION.fields_by_name['grant'].containing_oneof = _PERMISSIONSACTION.oneofs_by_name['action']
_PERMISSIONSACTION.oneofs_by_name['action'].fields.append(
  _PERMISSIONSACTION.fields_by_name['revoke'])
_PERMISSIONSACTION.fields_by_name['revoke'].containing_oneof = _PERMISSIONSACTION.oneofs_by_name['action']
_PERMISSIONSACTION.oneofs_by_name['action'].fields.append(
  _PERMISSIONSACTION.fields_by_name['set'])
_PERMISSIONSACTION.fields_by_name['set'].containing_oneof = _PERMISSIONSACTION.oneofs_by_name['action']
_PERMISSIONSACTION.oneofs_by_name['action'].fields.append(
  _PERMISSIONSACTION.fields_by_name['change_owner'])
_PERMISSIONSACTION.fields_by_name['change_owner'].containing_oneof = _PERMISSIONSACTION.oneofs_by_name['action']
_MODIFYPERMISSIONSREQUEST.fields_by_name['operation_params'].message_type = protos_dot_ydb__operation__pb2._OPERATIONPARAMS
_MODIFYPERMISSIONSREQUEST.fields_by_name['actions'].message_type = _PERMISSIONSACTION
_MODIFYPERMISSIONSREQUEST.oneofs_by_name['inheritance'].fields.append(
  _MODIFYPERMISSIONSREQUEST.fields_by_name['interrupt_inheritance'])
_MODIFYPERMISSIONSREQUEST.fields_by_name['interrupt_inheritance'].containing_oneof = _MODIFYPERMISSIONSREQUEST.oneofs_by_name['inheritance']
_MODIFYPERMISSIONSRESPONSE.fields_by_name['operation'].message_type = protos_dot_ydb__operation__pb2._OPERATION
DESCRIPTOR.message_types_by_name['MakeDirectoryRequest'] = _MAKEDIRECTORYREQUEST
DESCRIPTOR.message_types_by_name['MakeDirectoryResponse'] = _MAKEDIRECTORYRESPONSE
DESCRIPTOR.message_types_by_name['RemoveDirectoryRequest'] = _REMOVEDIRECTORYREQUEST
DESCRIPTOR.message_types_by_name['RemoveDirectoryResponse'] = _REMOVEDIRECTORYRESPONSE
DESCRIPTOR.message_types_by_name['ListDirectoryRequest'] = _LISTDIRECTORYREQUEST
DESCRIPTOR.message_types_by_name['ListDirectoryResponse'] = _LISTDIRECTORYRESPONSE
DESCRIPTOR.message_types_by_name['Permissions'] = _PERMISSIONS
DESCRIPTOR.message_types_by_name['Entry'] = _ENTRY
DESCRIPTOR.message_types_by_name['ListDirectoryResult'] = _LISTDIRECTORYRESULT
DESCRIPTOR.message_types_by_name['DescribePathRequest'] = _DESCRIBEPATHREQUEST
DESCRIPTOR.message_types_by_name['DescribePathResponse'] = _DESCRIBEPATHRESPONSE
DESCRIPTOR.message_types_by_name['DescribePathResult'] = _DESCRIBEPATHRESULT
DESCRIPTOR.message_types_by_name['PermissionsAction'] = _PERMISSIONSACTION
DESCRIPTOR.message_types_by_name['ModifyPermissionsRequest'] = _MODIFYPERMISSIONSREQUEST
DESCRIPTOR.message_types_by_name['ModifyPermissionsResponse'] = _MODIFYPERMISSIONSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MakeDirectoryRequest = _reflection.GeneratedProtocolMessageType('MakeDirectoryRequest', (_message.Message,), {
  'DESCRIPTOR' : _MAKEDIRECTORYREQUEST,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.MakeDirectoryRequest)
  })
_sym_db.RegisterMessage(MakeDirectoryRequest)

MakeDirectoryResponse = _reflection.GeneratedProtocolMessageType('MakeDirectoryResponse', (_message.Message,), {
  'DESCRIPTOR' : _MAKEDIRECTORYRESPONSE,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.MakeDirectoryResponse)
  })
_sym_db.RegisterMessage(MakeDirectoryResponse)

RemoveDirectoryRequest = _reflection.GeneratedProtocolMessageType('RemoveDirectoryRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEDIRECTORYREQUEST,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.RemoveDirectoryRequest)
  })
_sym_db.RegisterMessage(RemoveDirectoryRequest)

RemoveDirectoryResponse = _reflection.GeneratedProtocolMessageType('RemoveDirectoryResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEDIRECTORYRESPONSE,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.RemoveDirectoryResponse)
  })
_sym_db.RegisterMessage(RemoveDirectoryResponse)

ListDirectoryRequest = _reflection.GeneratedProtocolMessageType('ListDirectoryRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTDIRECTORYREQUEST,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.ListDirectoryRequest)
  })
_sym_db.RegisterMessage(ListDirectoryRequest)

ListDirectoryResponse = _reflection.GeneratedProtocolMessageType('ListDirectoryResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTDIRECTORYRESPONSE,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.ListDirectoryResponse)
  })
_sym_db.RegisterMessage(ListDirectoryResponse)

Permissions = _reflection.GeneratedProtocolMessageType('Permissions', (_message.Message,), {
  'DESCRIPTOR' : _PERMISSIONS,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.Permissions)
  })
_sym_db.RegisterMessage(Permissions)

Entry = _reflection.GeneratedProtocolMessageType('Entry', (_message.Message,), {
  'DESCRIPTOR' : _ENTRY,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.Entry)
  })
_sym_db.RegisterMessage(Entry)

ListDirectoryResult = _reflection.GeneratedProtocolMessageType('ListDirectoryResult', (_message.Message,), {
  'DESCRIPTOR' : _LISTDIRECTORYRESULT,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.ListDirectoryResult)
  })
_sym_db.RegisterMessage(ListDirectoryResult)

DescribePathRequest = _reflection.GeneratedProtocolMessageType('DescribePathRequest', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEPATHREQUEST,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.DescribePathRequest)
  })
_sym_db.RegisterMessage(DescribePathRequest)

DescribePathResponse = _reflection.GeneratedProtocolMessageType('DescribePathResponse', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEPATHRESPONSE,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.DescribePathResponse)
  })
_sym_db.RegisterMessage(DescribePathResponse)

DescribePathResult = _reflection.GeneratedProtocolMessageType('DescribePathResult', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEPATHRESULT,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.DescribePathResult)
  })
_sym_db.RegisterMessage(DescribePathResult)

PermissionsAction = _reflection.GeneratedProtocolMessageType('PermissionsAction', (_message.Message,), {
  'DESCRIPTOR' : _PERMISSIONSACTION,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.PermissionsAction)
  })
_sym_db.RegisterMessage(PermissionsAction)

ModifyPermissionsRequest = _reflection.GeneratedProtocolMessageType('ModifyPermissionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODIFYPERMISSIONSREQUEST,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.ModifyPermissionsRequest)
  })
_sym_db.RegisterMessage(ModifyPermissionsRequest)

ModifyPermissionsResponse = _reflection.GeneratedProtocolMessageType('ModifyPermissionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _MODIFYPERMISSIONSRESPONSE,
  '__module__' : 'protos.ydb_scheme_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Scheme.ModifyPermissionsResponse)
  })
_sym_db.RegisterMessage(ModifyPermissionsResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from protos import ydb_operation_pb2 as _ydb_operation_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ComputeNodeStatus(_message.Message):
    __slots__ = ["id", "load", "overall", "pools", "tablets"]
    ID_FIELD_NUMBER: _ClassVar[int]
    LOAD_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    POOLS_FIELD_NUMBER: _ClassVar[int]
    TABLETS_FIELD_NUMBER: _ClassVar[int]
    id: str
    load: LoadAverageStatus
    overall: StatusFlag.Status
    pools: _containers.RepeatedCompositeFieldContainer[ThreadPoolStatus]
    tablets: _containers.RepeatedCompositeFieldContainer[ComputeTabletStatus]
    def __init__(self, id: _Optional[str] = ..., overall: _Optional[_Union[StatusFlag.Status, str]] = ..., tablets: _Optional[_Iterable[_Union[ComputeTabletStatus, _Mapping]]] = ..., pools: _Optional[_Iterable[_Union[ThreadPoolStatus, _Mapping]]] = ..., load: _Optional[_Union[LoadAverageStatus, _Mapping]] = ...) -> None: ...

class ComputeStatus(_message.Message):
    __slots__ = ["nodes", "overall", "tablets"]
    NODES_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    TABLETS_FIELD_NUMBER: _ClassVar[int]
    nodes: _containers.RepeatedCompositeFieldContainer[ComputeNodeStatus]
    overall: StatusFlag.Status
    tablets: _containers.RepeatedCompositeFieldContainer[ComputeTabletStatus]
    def __init__(self, overall: _Optional[_Union[StatusFlag.Status, str]] = ..., nodes: _Optional[_Iterable[_Union[ComputeNodeStatus, _Mapping]]] = ..., tablets: _Optional[_Iterable[_Union[ComputeTabletStatus, _Mapping]]] = ...) -> None: ...

class ComputeTabletStatus(_message.Message):
    __slots__ = ["count", "id", "overall", "state", "type"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    count: int
    id: _containers.RepeatedScalarFieldContainer[str]
    overall: StatusFlag.Status
    state: str
    type: str
    def __init__(self, overall: _Optional[_Union[StatusFlag.Status, str]] = ..., type: _Optional[str] = ..., state: _Optional[str] = ..., count: _Optional[int] = ..., id: _Optional[_Iterable[str]] = ...) -> None: ...

class DatabaseStatus(_message.Message):
    __slots__ = ["compute", "name", "overall", "storage"]
    COMPUTE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    STORAGE_FIELD_NUMBER: _ClassVar[int]
    compute: ComputeStatus
    name: str
    overall: StatusFlag.Status
    storage: StorageStatus
    def __init__(self, name: _Optional[str] = ..., overall: _Optional[_Union[StatusFlag.Status, str]] = ..., storage: _Optional[_Union[StorageStatus, _Mapping]] = ..., compute: _Optional[_Union[ComputeStatus, _Mapping]] = ...) -> None: ...

class IssueLog(_message.Message):
    __slots__ = ["count", "id", "level", "listed", "location", "message", "reason", "status", "type"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    LISTED_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    count: int
    id: str
    level: int
    listed: int
    location: Location
    message: str
    reason: _containers.RepeatedScalarFieldContainer[str]
    status: StatusFlag.Status
    type: str
    def __init__(self, id: _Optional[str] = ..., status: _Optional[_Union[StatusFlag.Status, str]] = ..., message: _Optional[str] = ..., location: _Optional[_Union[Location, _Mapping]] = ..., reason: _Optional[_Iterable[str]] = ..., type: _Optional[str] = ..., level: _Optional[int] = ..., listed: _Optional[int] = ..., count: _Optional[int] = ...) -> None: ...

class LoadAverageStatus(_message.Message):
    __slots__ = ["cores", "load", "overall"]
    CORES_FIELD_NUMBER: _ClassVar[int]
    LOAD_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    cores: int
    load: float
    overall: StatusFlag.Status
    def __init__(self, overall: _Optional[_Union[StatusFlag.Status, str]] = ..., load: _Optional[float] = ..., cores: _Optional[int] = ...) -> None: ...

class Location(_message.Message):
    __slots__ = ["compute", "database", "storage"]
    COMPUTE_FIELD_NUMBER: _ClassVar[int]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    STORAGE_FIELD_NUMBER: _ClassVar[int]
    compute: LocationCompute
    database: LocationDatabase
    storage: LocationStorage
    def __init__(self, storage: _Optional[_Union[LocationStorage, _Mapping]] = ..., compute: _Optional[_Union[LocationCompute, _Mapping]] = ..., database: _Optional[_Union[LocationDatabase, _Mapping]] = ...) -> None: ...

class LocationCompute(_message.Message):
    __slots__ = ["node", "pool", "tablet"]
    NODE_FIELD_NUMBER: _ClassVar[int]
    POOL_FIELD_NUMBER: _ClassVar[int]
    TABLET_FIELD_NUMBER: _ClassVar[int]
    node: LocationNode
    pool: LocationComputePool
    tablet: LocationComputeTablet
    def __init__(self, node: _Optional[_Union[LocationNode, _Mapping]] = ..., pool: _Optional[_Union[LocationComputePool, _Mapping]] = ..., tablet: _Optional[_Union[LocationComputeTablet, _Mapping]] = ...) -> None: ...

class LocationComputePool(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class LocationComputeTablet(_message.Message):
    __slots__ = ["count", "id", "type"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    count: int
    id: _containers.RepeatedScalarFieldContainer[str]
    type: str
    def __init__(self, type: _Optional[str] = ..., id: _Optional[_Iterable[str]] = ..., count: _Optional[int] = ...) -> None: ...

class LocationDatabase(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class LocationNode(_message.Message):
    __slots__ = ["host", "id", "port"]
    HOST_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    PORT_FIELD_NUMBER: _ClassVar[int]
    host: str
    id: int
    port: int
    def __init__(self, id: _Optional[int] = ..., host: _Optional[str] = ..., port: _Optional[int] = ...) -> None: ...

class LocationStorage(_message.Message):
    __slots__ = ["node", "pool"]
    NODE_FIELD_NUMBER: _ClassVar[int]
    POOL_FIELD_NUMBER: _ClassVar[int]
    node: LocationNode
    pool: LocationStoragePool
    def __init__(self, node: _Optional[_Union[LocationNode, _Mapping]] = ..., pool: _Optional[_Union[LocationStoragePool, _Mapping]] = ...) -> None: ...

class LocationStorageGroup(_message.Message):
    __slots__ = ["id", "vdisk"]
    ID_FIELD_NUMBER: _ClassVar[int]
    VDISK_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    vdisk: LocationStorageVDisk
    def __init__(self, id: _Optional[_Iterable[str]] = ..., vdisk: _Optional[_Union[LocationStorageVDisk, _Mapping]] = ...) -> None: ...

class LocationStoragePDisk(_message.Message):
    __slots__ = ["id", "path"]
    ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    id: str
    path: str
    def __init__(self, id: _Optional[str] = ..., path: _Optional[str] = ...) -> None: ...

class LocationStoragePool(_message.Message):
    __slots__ = ["group", "name"]
    GROUP_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    group: LocationStorageGroup
    name: str
    def __init__(self, name: _Optional[str] = ..., group: _Optional[_Union[LocationStorageGroup, _Mapping]] = ...) -> None: ...

class LocationStorageVDisk(_message.Message):
    __slots__ = ["id", "pdisk"]
    ID_FIELD_NUMBER: _ClassVar[int]
    PDISK_FIELD_NUMBER: _ClassVar[int]
    id: _containers.RepeatedScalarFieldContainer[str]
    pdisk: _containers.RepeatedCompositeFieldContainer[LocationStoragePDisk]
    def __init__(self, id: _Optional[_Iterable[str]] = ..., pdisk: _Optional[_Iterable[_Union[LocationStoragePDisk, _Mapping]]] = ...) -> None: ...

class NodeCheckRequest(_message.Message):
    __slots__ = ["operation_params"]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ...) -> None: ...

class NodeCheckResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class SelfCheck(_message.Message):
    __slots__ = []
    class Result(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    DEGRADED: SelfCheck.Result
    EMERGENCY: SelfCheck.Result
    GOOD: SelfCheck.Result
    MAINTENANCE_REQUIRED: SelfCheck.Result
    UNSPECIFIED: SelfCheck.Result
    def __init__(self) -> None: ...

class SelfCheckRequest(_message.Message):
    __slots__ = ["maximum_level", "minimum_status", "operation_params", "return_verbose_status"]
    MAXIMUM_LEVEL_FIELD_NUMBER: _ClassVar[int]
    MINIMUM_STATUS_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    RETURN_VERBOSE_STATUS_FIELD_NUMBER: _ClassVar[int]
    maximum_level: int
    minimum_status: StatusFlag.Status
    operation_params: _ydb_operation_pb2.OperationParams
    return_verbose_status: bool
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., return_verbose_status: bool = ..., minimum_status: _Optional[_Union[StatusFlag.Status, str]] = ..., maximum_level: _Optional[int] = ...) -> None: ...

class SelfCheckResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class SelfCheckResult(_message.Message):
    __slots__ = ["database_status", "issue_log", "self_check_result"]
    DATABASE_STATUS_FIELD_NUMBER: _ClassVar[int]
    ISSUE_LOG_FIELD_NUMBER: _ClassVar[int]
    SELF_CHECK_RESULT_FIELD_NUMBER: _ClassVar[int]
    database_status: _containers.RepeatedCompositeFieldContainer[DatabaseStatus]
    issue_log: _containers.RepeatedCompositeFieldContainer[IssueLog]
    self_check_result: SelfCheck.Result
    def __init__(self, self_check_result: _Optional[_Union[SelfCheck.Result, str]] = ..., issue_log: _Optional[_Iterable[_Union[IssueLog, _Mapping]]] = ..., database_status: _Optional[_Iterable[_Union[DatabaseStatus, _Mapping]]] = ...) -> None: ...

class StatusFlag(_message.Message):
    __slots__ = []
    class Status(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    BLUE: StatusFlag.Status
    GREEN: StatusFlag.Status
    GREY: StatusFlag.Status
    ORANGE: StatusFlag.Status
    RED: StatusFlag.Status
    UNSPECIFIED: StatusFlag.Status
    YELLOW: StatusFlag.Status
    def __init__(self) -> None: ...

class StorageGroupStatus(_message.Message):
    __slots__ = ["id", "overall", "vdisks"]
    ID_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    VDISKS_FIELD_NUMBER: _ClassVar[int]
    id: str
    overall: StatusFlag.Status
    vdisks: _containers.RepeatedCompositeFieldContainer[StorageVDiskStatus]
    def __init__(self, id: _Optional[str] = ..., overall: _Optional[_Union[StatusFlag.Status, str]] = ..., vdisks: _Optional[_Iterable[_Union[StorageVDiskStatus, _Mapping]]] = ...) -> None: ...

class StoragePDiskStatus(_message.Message):
    __slots__ = ["id", "overall"]
    ID_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    id: str
    overall: StatusFlag.Status
    def __init__(self, id: _Optional[str] = ..., overall: _Optional[_Union[StatusFlag.Status, str]] = ...) -> None: ...

class StoragePoolStatus(_message.Message):
    __slots__ = ["groups", "id", "overall"]
    GROUPS_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    groups: _containers.RepeatedCompositeFieldContainer[StorageGroupStatus]
    id: str
    overall: StatusFlag.Status
    def __init__(self, id: _Optional[str] = ..., overall: _Optional[_Union[StatusFlag.Status, str]] = ..., groups: _Optional[_Iterable[_Union[StorageGroupStatus, _Mapping]]] = ...) -> None: ...

class StorageStatus(_message.Message):
    __slots__ = ["overall", "pools"]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    POOLS_FIELD_NUMBER: _ClassVar[int]
    overall: StatusFlag.Status
    pools: _containers.RepeatedCompositeFieldContainer[StoragePoolStatus]
    def __init__(self, overall: _Optional[_Union[StatusFlag.Status, str]] = ..., pools: _Optional[_Iterable[_Union[StoragePoolStatus, _Mapping]]] = ...) -> None: ...

class StorageVDiskStatus(_message.Message):
    __slots__ = ["id", "overall", "pdisk", "vdisk_status"]
    ID_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    PDISK_FIELD_NUMBER: _ClassVar[int]
    VDISK_STATUS_FIELD_NUMBER: _ClassVar[int]
    id: str
    overall: StatusFlag.Status
    pdisk: StoragePDiskStatus
    vdisk_status: StatusFlag.Status
    def __init__(self, id: _Optional[str] = ..., overall: _Optional[_Union[StatusFlag.Status, str]] = ..., vdisk_status: _Optional[_Union[StatusFlag.Status, str]] = ..., pdisk: _Optional[_Union[StoragePDiskStatus, _Mapping]] = ...) -> None: ...

class ThreadPoolStatus(_message.Message):
    __slots__ = ["name", "overall", "usage"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OVERALL_FIELD_NUMBER: _ClassVar[int]
    USAGE_FIELD_NUMBER: _ClassVar[int]
    name: str
    overall: StatusFlag.Status
    usage: float
    def __init__(self, overall: _Optional[_Union[StatusFlag.Status, str]] = ..., name: _Optional[str] = ..., usage: _Optional[float] = ...) -> None: ...



import sys

import google.protobuf
from packaging.version import Version
from ... import _utilities

# generated files are incompatible between 3 and 4 protobuf versions
# import right generated version for current protobuf lib
# sdk code must always import from ydb._grpc.common
protobuf_version = Version(google.protobuf.__version__)

# for compatible with arcadia
if _utilities.check_module_exists("contrib.ydb.public.api"):
    from contrib.ydb.public.api.grpc import *  # noqa

    sys.modules["ydb._grpc.common"] = sys.modules["contrib.ydb.public.api.grpc"]

    from contrib.ydb.public.api import protos

    sys.modules["ydb._grpc.common.protos"] = sys.modules["contrib.ydb.public.api.protos"]
elif _utilities.check_module_exists("ydb.public.api"):
    from ydb.public.api.grpc import *  # noqa

    sys.modules["ydb._grpc.common"] = sys.modules["ydb.public.api.grpc"]

    from ydb.public.api import protos

    sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb.public.api.protos"]
else:
    # common way, outside of arcadia
    if protobuf_version < Version("4.0"):
        from ydb._grpc.v3 import *  # noqa

        sys.modules["ydb._grpc.common"] = sys.modules["ydb._grpc.v3"]

        from ydb._grpc.v3 import protos  # noqa

        sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v3.protos"]
    elif protobuf_version < Version("5.0"):
        from ydb._grpc.v4 import *  # noqa

        sys.modules["ydb._grpc.common"] = sys.modules["ydb._grpc.v4"]

        from ydb._grpc.v4 import protos  # noqa

        sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v4.protos"]

    elif protobuf_version < Version("6.0"):
        from ydb._grpc.v5 import *  # noqa

        sys.modules["ydb._grpc.common"] = sys.modules["ydb._grpc.v5"]

        from ydb._grpc.v5 import protos  # noqa

        sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v5.protos"]

    else:
        from ydb._grpc.v6 import *  # noqa

        sys.modules["ydb._grpc.common"] = sys.modules["ydb._grpc.v6"]

        from ydb._grpc.v6 import protos  # noqa

        sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v6.protos"]
